@@ -1,8 +1,8 @@
 package Cassandra::Client;
 our $AUTHORITY = 'cpan:TVDW';
-$Cassandra::Client::VERSION = '0.12_001'; # TRIAL
+$Cassandra::Client::VERSION = '0.13_001'; # TRIAL
 
-$Cassandra::Client::VERSION = '0.12001';# ABSTRACT: Perl library for accessing Cassandra using its binary network protocol
+$Cassandra::Client::VERSION = '0.13001';# ABSTRACT: Perl library for accessing Cassandra using its binary network protocol
 
 use 5.010;
 use strict;
@@ -16,7 +16,8 @@ use Cassandra::Client::Metadata;
 use Cassandra::Client::Policy::Queue::Default;
 use Cassandra::Client::Policy::Retry::Default;
 use Cassandra::Client::Policy::Retry;
-use Cassandra::Client::Policy::Throttle::Adaptive;
+use Cassandra::Client::Policy::Throttle::Default;
+use Cassandra::Client::Policy::LoadBalancing::Default;
 use Cassandra::Client::Pool;
 use Cassandra::Client::TLSHandling;
 use Cassandra::Client::Util qw/series whilst/;
@@ -47,6 +48,12 @@ sub new {
     my $options= Cassandra::Client::Config->new(
         \%args
     );
+
+    $self->{throttler}= $options->{throttler} || Cassandra::Client::Policy::Throttle::Default->new();
+    $self->{retry_policy}= $options->{retry_policy} || Cassandra::Client::Policy::Retry::Default->new();
+    $self->{command_queue}= $options->{command_queue} || Cassandra::Client::Policy::Queue::Default->new();
+    $self->{load_balancing_policy}= $options->{load_balancing_policy} || Cassandra::Client::Policy::LoadBalancing::Default->new();
+
     my $async_class= $options->{anyevent} ? "Cassandra::Client::AsyncAnyEvent" : "Cassandra::Client::AsyncEV";
     my $async_io= $async_class->new(
         options => $options,
@@ -59,27 +66,15 @@ sub new {
         options  => $options,
         metadata => $metadata,
         async_io => $async_io,
+        load_balancing_policy => $self->{load_balancing_policy},
     );
-    my $command_queue= Cassandra::Client::Policy::Queue::Default->new(
-        %{ $options->{command_queue_config} || {} },
-    );
-    my $retry_policy= Cassandra::Client::Policy::Retry::Default->new();
     my $tls= $options->{tls} ? Cassandra::Client::TLSHandling->new() : undef;
 
     $self->{options}= $options;
     $self->{async_io}= $async_io;
     $self->{metadata}= $metadata;
     $self->{pool}= $pool;
-    $self->{command_queue}= $command_queue;
-    $self->{retry_policy}= $retry_policy;
     $self->{tls}= $tls;
-    if ($options->{throttler}) {
-        $options->{throttler}= 'Adaptive' if $options->{throttler} eq 'AdaptiveThrottler'; # Temporary.
-        my $throttler_class= "Cassandra::Client::Policy::Throttle::$options->{throttler}";
-        $options->{throttler}= $throttler_class->new(%{$options->{throttler_config}});
-    } else {
-        $self->{throttler}= undef;
-    }
 
     return $self;
 }
@@ -234,7 +229,7 @@ sub _command {
     };
 
     # Handle overloads
-    goto FAILFAST if $self->{throttler} && $self->{throttler}->should_fail();
+    goto FAILFAST if $self->{throttler}->should_fail();
 
     goto OVERFLOW if $self->{active_queries} >= $self->{options}{max_concurrent_queries};
 
@@ -246,9 +241,7 @@ sub _command {
     $self->{active_queries}++;
     $connection->$command(sub {
         my ($error, $result)= @_;
-        if (my $throttler= $self->{throttler}) {
-            $throttler->count($error);
-        }
+        $self->{throttler}->count($error);
 
         $self->{active_queries}--;
         $self->_command_dequeue if $self->{command_queue}{has_any};
@@ -286,16 +279,14 @@ sub _command_slowpath {
             # Yes, if we immediately take the slow path, which we would if we're not connected, we're going to throttle twice
             # For now, I'm okay with that, but let's mark it with a TODO anyway.
             # XXX
-            if ($self->{throttler} && $self->{throttler}->should_fail()) {
+            if ($self->{throttler}->should_fail()) {
                 return $next->("Client-induced failure by throttling mechanism");
             }
             $connection->$command($next, @$args);
         }
     ], sub {
         my ($error, $result)= @_;
-        if (my $throttler= $self->{throttler}) {
-            $throttler->count($error);
-        }
+        $self->{throttler}->count($error);
 
         $self->{active_queries}--;
         $self->_command_dequeue if $self->{command_queue}{has_any};
@@ -327,16 +318,16 @@ sub _command_failed {
     return $callback->($error) unless is_ref($error);
 
     my $retry_decision;
-    if ($error->{do_retry}) {
+    if ($error->do_retry) {
         $retry_decision= Cassandra::Client::Policy::Retry::retry;
-    } elsif ($error->{request_error}) {
+    } elsif ($error->is_request_error) {
         $retry_decision= $self->{retry_policy}->on_request_error(undef, undef, $error, ($command_info->{retries}||0));
-    } elsif ($error->code == 0x1100) {
-        $retry_decision= $self->{retry_policy}->on_write_timeout(undef, @$error{qw/cl write_type blockfor received/}, ($command_info->{retries}||0));
-    } elsif ($error->code == 0x1200) {
-        $retry_decision= $self->{retry_policy}->on_read_timeout(undef, @$error{qw/cl blockfor received data_retrieved/}, ($command_info->{retries}||0));
-    } elsif ($error->code == 0x1000) {
-        $retry_decision= $self->{retry_policy}->on_unavailable(undef, @$error{qw/cl required alive/}, ($command_info->{retries}||0));
+    } elsif ($error->isa('Cassandra::Client::Error::WriteTimeoutException')) {
+        $retry_decision= $self->{retry_policy}->on_write_timeout(undef, $error->cl, $error->write_type, $error->blockfor, $error->received, ($command_info->{retries}||0));
+    } elsif ($error->isa('Cassandra::Client::Error::ReadTimeoutException')) {
+        $retry_decision= $self->{retry_policy}->on_read_timeout(undef, $error->cl, $error->blockfor, $error->received, $error->data_retrieved, ($command_info->{retries}||0));
+    } elsif ($error->isa('Cassandra::Client::Error::UnavailableException')) {
+        $retry_decision= $self->{retry_policy}->on_unavailable(undef, $error->cl, $error->required, $error->alive, ($command_info->{retries}||0));
     } else {
         $retry_decision= Cassandra::Client::Policy::Retry::rethrow;
     }
@@ -533,7 +524,7 @@ Cassandra::Client - Perl library for accessing Cassandra using its binary networ
 
 =head1 VERSION
 
-version 0.12_001
+version 0.13_001
 
 =head1 DESCRIPTION
 
