@@ -1,8 +1,9 @@
 package Cassandra::Client::Policy::Throttle::Adaptive;
 our $AUTHORITY = 'cpan:TVDW';
-$Cassandra::Client::Policy::Throttle::Adaptive::VERSION = '0.12_001'; # TRIAL
+$Cassandra::Client::Policy::Throttle::Adaptive::VERSION = '0.13_001'; # TRIAL
 
-$Cassandra::Client::Policy::Throttle::Adaptive::VERSION = '0.12001';use 5.010;
+$Cassandra::Client::Policy::Throttle::Adaptive::VERSION = '0.13001';use parent 'Cassandra::Client::Policy::Throttle::Default';
+use 5.010;
 use strict;
 use warnings;
 use Time::HiRes qw/CLOCK_MONOTONIC/;
@@ -45,7 +46,7 @@ sub should_fail {
 sub count {
     my ($self, $error, $force_error)= @_;
     $self->{window_total}++;
-    my $success= !(is_blessed_ref($error) && $error->{is_timeout}) && !$force_error;
+    my $success= !(is_blessed_ref($error) && $error->is_timeout) && !$force_error;
     push @{$self->{window}}, [ Time::HiRes::clock_gettime(CLOCK_MONOTONIC)+$self->{time}, $success ];
     $self->{window_success}++ if $success;
     return;
@@ -63,7 +64,7 @@ Cassandra::Client::Policy::Throttle::Adaptive
 
 =head1 VERSION
 
-version 0.12_001
+version 0.13_001
 
 =head1 AUTHOR
 
