@@ -1,14 +1,13 @@
 package Cassandra::Client::Pool;
 our $AUTHORITY = 'cpan:TVDW';
-$Cassandra::Client::Pool::VERSION = '0.12_001'; # TRIAL
+$Cassandra::Client::Pool::VERSION = '0.13_001'; # TRIAL
 
-$Cassandra::Client::Pool::VERSION = '0.12001';use 5.010;
+$Cassandra::Client::Pool::VERSION = '0.13001';use 5.010;
 use strict;
 use warnings;
 
 use Scalar::Util 'weaken';
 use Cassandra::Client::Util;
-use Cassandra::Client::Policy::LoadBalancing::Default;
 use Cassandra::Client::NetworkStatus;
 
 sub new {
@@ -19,7 +18,7 @@ sub new {
         metadata => $args{metadata},
         max_connections => $args{options}{max_connections},
         async_io => $args{async_io},
-        policy => Cassandra::Client::Policy::LoadBalancing::Default->new(),
+        policy => $args{load_balancing_policy},
 
         shutdown => 0,
         pool => {},
@@ -238,7 +237,7 @@ sub spawn_new_connection {
 
                 for my $waiter (@$waiters) {
                     if ((++$waiter->{attempts}) >= $max_attempts || !%{$self->{connecting}}) {
-                        $waiter->{callback}->("Failed to connect to server");
+                        $waiter->{callback}->("Failed to connect to server: $error");
                     } else {
                         push @{$self->{wait_connect} ||= []}, $waiter;
                     }
@@ -294,7 +293,7 @@ Cassandra::Client::Pool
 
 =head1 VERSION
 
-version 0.12_001
+version 0.13_001
 
 =head1 AUTHOR
 
