@@ -1,8 +1,8 @@
 package Cassandra::Client::Protocol;
 our $AUTHORITY = 'cpan:TVDW';
-$Cassandra::Client::Protocol::VERSION = '0.12_001'; # TRIAL
+$Cassandra::Client::Protocol::VERSION = '0.13_001'; # TRIAL
 
-$Cassandra::Client::Protocol::VERSION = '0.12001';use 5.010;
+$Cassandra::Client::Protocol::VERSION = '0.13001';use 5.010;
 use strict;
 use warnings;
 
@@ -11,7 +11,10 @@ use Encode;
 require Exporter;
 our @ISA= qw(Exporter);
 
-use Cassandra::Client::Error;
+use Cassandra::Client::Error::Base;
+use Cassandra::Client::Error::ReadTimeoutException;
+use Cassandra::Client::Error::WriteTimeoutException;
+use Cassandra::Client::Error::UnavailableException;
 
 use constant BIGINT_SUPPORTED => eval { unpack('q>', "\0\0\0\0\0\0\0\1") };
 use if !BIGINT_SUPPORTED, 'Math::BigInt';
@@ -87,19 +90,18 @@ BEGIN {
         keys %constants,
         qw/
             pack_int                unpack_int
-            pack_long               unpack_long
+            pack_long
             pack_short              unpack_short
             pack_string             unpack_string
-            pack_longstring         unpack_longstring
+            pack_longstring
             pack_stringlist         unpack_stringlist
             pack_bytes              unpack_bytes
             pack_shortbytes         unpack_shortbytes
-            pack_option_type        unpack_option_type
-            pack_optionlist_type    unpack_optionlist_type
-            pack_stringmap          unpack_stringmap
+            pack_option_type
+            pack_stringmap
             pack_stringmultimap     unpack_stringmultimap
-            pack_inet               unpack_inet
-            pack_char               unpack_char
+                                    unpack_inet
+                                    unpack_char
 
             pack_metadata           unpack_metadata
                                     unpack_errordata
@@ -109,7 +111,6 @@ BEGIN {
             %batch_type_lookup
 
             BIGINT_SUPPORTED
-                                    unpack_metadata2
         /
     );
 
@@ -133,30 +134,6 @@ BEGIN {
     constant->import( { %constants } );
 }
 
-my %java_type_to_ours= (
-    # Because of implementation details, we can only have non-complex types here, eg. int and blob
-    'org.apache.cassandra.db.marshal.UTF8Type'          => TYPE_VARCHAR,
-    'org.apache.cassandra.db.marshal.AsciiType'         => TYPE_ASCII,
-    'org.apache.cassandra.db.marshal.UUIDType'          => TYPE_UUID,
-    'org.apache.cassandra.db.marshal.TimeUUIDType'      => TYPE_TIMEUUID,
-    'org.apache.cassandra.db.marshal.Int32Type'         => TYPE_INT,
-    'org.apache.cassandra.db.marshal.BytesType'         => TYPE_BLOB,
-    'org.apache.cassandra.db.marshal.FloatType'         => TYPE_FLOAT,
-    'org.apache.cassandra.db.marshal.DoubleType'        => TYPE_DOUBLE,
-    'org.apache.cassandra.db.marshal.BooleanType'       => TYPE_BOOLEAN,
-    'org.apache.cassandra.db.marshal.InetAddressType'   => TYPE_INET,
-    'org.apache.cassandra.db.marshal.SimpleDateType'    => TYPE_DATE,
-    'org.apache.cassandra.db.marshal.TimeType'          => TYPE_TIME,
-    'org.apache.cassandra.db.marshal.ShortType'         => TYPE_SMALLINT,
-    'org.apache.cassandra.db.marshal.ByteType'          => TYPE_TINYINT,
-    'org.apache.cassandra.db.marshal.DateType'          => TYPE_TIMESTAMP,
-    'org.apache.cassandra.db.marshal.TimestampType'     => TYPE_TIMESTAMP,
-    'org.apache.cassandra.db.marshal.LongType'          => TYPE_BIGINT,
-    'org.apache.cassandra.db.marshal.DecimalType'       => TYPE_DECIMAL,
-    'org.apache.cassandra.db.marshal.IntegerType'       => TYPE_VARINT,
-    'org.apache.cassandra.db.marshal.CounterColumnType' => TYPE_COUNTER,
-);
-
 # TYPE: int
 sub pack_int {
     pack('l>', $_[0])
@@ -175,14 +152,6 @@ sub pack_long {
     }
 }
 
-sub unpack_long {
-    if (BIGINT_SUPPORTED) {
-        return unpack('q>', substr $_[0], 0, 8, '');
-    } else {
-        return bytes_to_bigint(substr($_[0], 0, 8, ''));
-    }
-}
-
 # TYPE: short
 sub pack_short {
     pack('n', $_[0])
@@ -193,10 +162,6 @@ sub unpack_short {
 }
 
 # TYPE: char
-sub pack_char {
-    pack('c', $_[0])
-}
-
 sub unpack_char {
     unpack('c', substr $_[0], 0, 1, '')
 }
@@ -234,17 +199,6 @@ sub pack_longstring {
     return pack('l>/a', $_[0]);
 }
 
-sub unpack_longstring {
-    my $length= &unpack_int;
-    if ($length > 0) {
-        my $string= substr($_[0], 0, $length, '');
-        utf8::decode $string;
-        return $string;
-    } else {
-        return '';
-    }
-}
-
 # TYPE: stringlist
 sub pack_stringlist {
     pack_short(0+@{$_[0]}).join('', map { pack_string($_) } @{$_[0]})
@@ -300,10 +254,6 @@ sub unpack_shortbytes {
 }
 
 # TYPE: inet
-sub pack_inet {
-    die 'Not implemented';
-}
-
 sub unpack_inet {
     my $length= unpack('C', substr($_[0], 0, 1, ''));
     my $tmp_val= substr($_[0], 0, $length, '');
@@ -351,57 +301,6 @@ sub pack_option_type {
     }
 }
 
-sub unpack_option_type {
-    my $id= &unpack_short;
-    my @value;
-    if ($id == TYPE_CUSTOM) {
-        @value= (&unpack_string);
-        # Java will sometimes send its types as CUSTOM instead of the real type. Quite annoying.
-        if (my $real_type= $java_type_to_ours{$value[0]}) {
-            $id= $real_type;
-            @value= ();
-        }
-    } elsif ($id < 0x20) {
-        # Nothing
-    } elsif ($id == TYPE_LIST || $id == TYPE_SET) { # List<?> / Set<?>
-        @value= (&unpack_option_type);
-    } elsif ($id == TYPE_MAP) { # Map<?,?>
-        @value= (&unpack_option_type, &unpack_option_type);
-    } elsif ($id == TYPE_UDT) {
-        my $keyspace= &unpack_string;
-        my $udt_name= &unpack_string;
-        my $field_n=  &unpack_short;
-        my @fields;
-        for (1..$field_n) {
-            my $name= &unpack_string;
-            my $type= &unpack_option_type;
-            push @fields, [ $name, $type ];
-        }
-        @value= ($keyspace, $udt_name, \@fields);
-    } elsif ($id == TYPE_TUPLE) { # Tuple
-        my $field_n= &unpack_short;
-        my @fields;
-        for (1..$field_n) {
-            push @fields, &unpack_option_type;
-        }
-        @value= (\@fields);
-    } else {
-        die 'Unable to decode protocol: no idea what type '.$id.' is...';
-    }
-
-    return [ $id, @value ];
-}
-
-# TYPE: optionlist_type
-sub pack_optionlist_type {
-    pack_short(0+@$_[0]).join('', map pack_option_type($_), @$_[0])
-}
-
-sub unpack_optionlist_type {
-    my $count= &unpack_short;
-    [ map &unpack_option_type, 1..$count ]
-}
-
 # TYPE: stringmap
 sub pack_stringmap {
     my $pairs= '';
@@ -413,16 +312,6 @@ sub pack_stringmap {
     return pack_short($count).$pairs;
 }
 
-sub unpack_stringmap {
-    my $count= &unpack_short;
-    my $result= {};
-    for (1..$count) {
-        my $key= &unpack_string;
-        $result->{$key}= &unpack_string;
-    }
-    return $result;
-}
-
 # TYPE: stringmultimap
 sub pack_stringmultimap {
     my $pairs= '';
@@ -465,40 +354,6 @@ sub pack_metadata {
     return $out;
 }
 
-sub unpack_metadata {
-    my ($flags, $columns_count, $paging_state, $keyspace_name, $table_name, @columns);
-
-    ($flags, $columns_count)= unpack('l>l>', substr($_[0], 0, 8, '')); # Short-circuited for perf
-    if ($flags & 2) {
-        $paging_state= &unpack_bytes;
-    }
-    unless ($flags & 4) { # "No metadata"
-        my $global_tables_spec= ($flags & 1);
-        if ($global_tables_spec) {
-            $keyspace_name= &unpack_string;
-            $table_name= &unpack_string;
-        }
-
-        for (1..$columns_count) {
-            my ($keyspace, $table);
-            if ($global_tables_spec) {
-                ($keyspace, $table)= ($keyspace_name, $table_name);
-            } else {
-                ($keyspace, $table)= (&unpack_string, &unpack_string);
-            }
-            my $column_name= &unpack_string;
-            my $type= &unpack_option_type;
-
-            push @columns, [ $keyspace, $table, $column_name, $type ];
-        }
-    }
-
-    return {
-        columns => ($flags & 4) ? undef : \@columns,
-        paging_state => $paging_state,
-    };
-}
-
 # Query parameters
 sub pack_queryparameters {
     my ($consistency, $skip_metadata, $page_size, $paging_state, $timestamp, $row)= @_;
@@ -535,21 +390,24 @@ sub unpack_errordata {
         $error{cl}= &unpack_short;
         $error{required}= &unpack_int;
         $error{alive}= &unpack_int;
+        return Cassandra::Client::Error::UnavailableException->new(%error);
     } elsif ($code == 0x1100) {
         # Write timeout
         $error{cl}= &unpack_short;
         $error{received}= &unpack_int;
         $error{blockfor}= &unpack_int;
         $error{write_type}= &unpack_string;
+        return Cassandra::Client::Error::WriteTimeoutException->new(%error);
     } elsif ($code == 0x1200) {
         # Read timeout
         $error{cl}= &unpack_short;
         $error{received}= &unpack_int;
         $error{blockfor}= &unpack_int;
         $error{data_present}= &unpack_char;
+        return Cassandra::Client::Error::ReadTimeoutException->new(%error);
     }
 
-    return Cassandra::Client::Error->new(%error);
+    return Cassandra::Client::Error::Base->new(%error);
 }
 
 # Support for 32bit perl
@@ -574,21 +432,6 @@ sub bigint_to_bytes {
     return $bytes;
 }
 
-sub bytes_to_bigint {
-    my $bytes= substr("\0\0\0\0\0\0\0\0".$_[0], -8);
-    my $negative= 0;
-    if ((substr($bytes, 0, 1) & "\x80") ne "\0") { # Negative
-        $negative= 1;
-        $bytes= ~$bytes;
-    }
-    my $mb= Math::BigInt->new('0x'.substr(("0"x16).unpack('H*', $bytes), -16));
-    if ($negative) {
-        $mb += 1; # Because two's complement
-        $mb *= -1;
-    }
-    return $mb->bstr;
-}
-
 1;
 
 __END__
@@ -601,7 +444,7 @@ Cassandra::Client::Protocol
 
 =head1 VERSION
 
-version 0.12_001
+version 0.13_001
 
 =head1 AUTHOR
 
